#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
import json
from sys import prefix
import click
import requests
import oss2
from pathlib import Path
from opendatalab.exception import OpenDataLabAuthenticationError, OpenDataLabError
from opendatalab.utils import bytes2human


class OpenDataLabAPI(object):
    def __init__(self, host, token):
        self.host = host
        self.token = token

    def get_dataset_sts(self, dataset):
        """Get dataset sts by dataset_name
        Args:
            dataset (string): dataset_name

        Raises:
            OpenDataLabError: 

        Returns:
            string: json data from response
        """
        resp = requests.get(
            f"{self.host}/api/datasets/{dataset}/sts",
            headers={"X-OPENDATALAB-API-TOKEN": self.token},
        )
        if resp.status_code != 200:
            raise OpenDataLabError(resp.status_code, resp.text)
        return resp.json()["data"]
    
    
    def login(self, username: str, password: str):
        data = {
            "email": username,
            "password": password,
        }
        data = json.dumps(data)
        resp = requests.post(
            f"{self.host}/api/users/login",
            data = data,
            headers={"Content-Type": "application/json"},
        )
        if resp.status_code != 200:
            raise OpenDataLabAuthenticationError(resp.status_code, resp.text)

        cookies_dict = requests.utils.dict_from_cookiejar(resp.cookies)
        if 'opendatalab_session' in cookies_dict.keys():
            opendatalab_session	= cookies_dict['opendatalab_session']
        else:
            raise OpenDataLabError(resp.status_code, "No opendatalab_session")
        
        config_json = {
            'endpoint': self.host,
            'user.email': username,
            'user.token': opendatalab_session,
            } 
        # click.echo(f"config_json: {config_json}")
        
        return config_json
        
    
    def ls(self, ds_path):
        ds_split = ds_path.split("/")
        if len(ds_split) > 1:
            dataset_name = ds_split[0]
        else:
            dataset_name = ds_path
        
        # dataset = Dataset(f"{self.host}/datasets/{dataset_name}", self.token)
        # bucket = dataset.get_oss_bucket()
        
        # if ds_path.endswith("/"):
        #     ds_path_prefix = ds_path
        # else:
        #     ds_path_prefix = ds_path + '/'

        # prefix = dataset.get_object_key_prefix() + ds_path_prefix
        # object_info_dict = {}
        # for info in oss2.ObjectIteratorV2(bucket, prefix):
        #     object_info_dict[info.key] = bytes2human(info.size)
            
        # sorted_object_info_dict = dict(sorted(object_info_dict.items(), key=lambda x: x[0], reverse=True))
        # print(f"FILE\t\tSIZE")
        # for key, val in sorted_object_info_dict.items():
        #     print(f"{key}\t\t{val}")
            
                    
    
    def search_dataset(self, dataset):
        resp = requests.get(
            f"{self.host}/api/datasets/?pageSize=25&keywords={dataset}",
            headers={"X-OPENDATALAB-API-TOKEN": self.token},
        )
        if resp.status_code != 200:
            raise OpenDataLabError(resp.status_code, resp.text)
        result_list = resp.json()["data"]["list"]
        print('=====index\t name\t fileBytes\t introduction\t downloadCount=====')
        if result_list:
            for i, res in enumerate(result_list):
                index = i
                ds_name = res['name']
                ds_desc = res['introductionText'][:30] + '...'
                ds_dw_cnt = res['downloadCount']
                ds_file_bytes = bytes2human(res['fileBytes'])
                print(f"{index}\t {ds_name}\t {ds_file_bytes}\t {ds_desc}\t {ds_dw_cnt}")
        
        return result_list

    
    def get_info(self, dataset):
        resp = requests.get(
            f"{self.host}/api/datasets/{dataset}",
            headers={"X-OPENDATALAB-API-TOKEN": self.token},
        )
        if resp.status_code != 200:
            raise OpenDataLabError(resp.status_code, resp.text)
        data =  resp.json()["data"]
        info_data = {
            'id': data['id'],
            'name': data['name'],
            'fileBytes': bytes2human(data['fileBytes']),
            'fileCount': data['fileCount'],
            'publishDate': data['publishDate'],
            'licenses': data['licenses'],
            'publisher': data['publisher'],
            'labelFileTypes': data['labelFileTypes'],
            'mediaTypes': data['mediaTypes'],
            'labelTypes': data['labelTypes'],            
        }
        print(json.dumps(info_data, indent=4, sort_keys=True))
                
        return info_data